from ..module.termcolor import colored
from ..module.LoggyTheme import LoggyTheme
from ..module.LogLevelEnum import LogLevel


class ClassicTheme(LoggyTheme):

    def apply_theme(self, log_level, log, *texts, name=None, show_log_time=False, log_time_string=""):
        render_log = ""
        if show_log_time:
            render_log += self.render_log_time(log_level, log_time_string)
            render_log += " "
        render_log += self.render_name(log_level, name)
        if name is not None:
            render_log += colored(" ", color="green")
        render_log += self.render_log_level(log_level)
        render_log += " "
        render_log += self.render_log_text(log_level, log, *texts)

        return render_log

    def theme_name(self, log_level, name):
        render_name = ""
        if log_level == LogLevel.INFO:
            render_name = colored(" " + name + " ", color="green", attrs=['reverse', 'blink', 'bold'])
        elif log_level == LogLevel.DEBUG:
            render_name = colored(" " + name + " ", color="blue", attrs=['reverse', 'blink', 'bold'])
        elif log_level == LogLevel.WARNING:
            render_name = colored(" " + name + " ", color="yellow", attrs=['reverse', 'blink', 'bold'])
        elif log_level == LogLevel.ERROR:
            render_name = colored(" " + name + " ", color="red", attrs=['reverse', 'blink', 'bold'])
        elif log_level == LogLevel.CRITICAL:
            render_name = colored(" " + name + " ", color="magenta", attrs=['reverse', 'blink', 'bold'])

        return render_name

    def theme_log_time(self, log_level, log_time_text):
        render_log = ""
        if log_level is LogLevel.INFO:
            render_log = colored(" " + log_time_text + " ", color="green", attrs=['reverse', 'blink', 'bold'])
        elif log_level is LogLevel.DEBUG:
            render_log = colored(" " + log_time_text + " ", color="blue", attrs=['reverse', 'blink', 'bold'])
        elif log_level is LogLevel.WARNING:
            render_log = colored(" " + log_time_text + " ", color="yellow", attrs=['reverse', 'blink', 'bold'])
        elif log_level is LogLevel.ERROR:
            render_log = colored(" " + log_time_text + " ", color="red", attrs=['reverse', 'blink', 'bold'])
        elif log_level is LogLevel.CRITICAL:
            render_log = colored(" " + log_time_text + " ", color="magenta", attrs=['reverse', 'blink', 'bold'])

        return render_log

    def theme_log_level(self, log_level):
        render_log_level = ""
        if log_level == LogLevel.INFO:
            render_log_level = colored("[" + log_level.name + "]", color="green", attrs=['blink', 'bold'])
        elif log_level == LogLevel.DEBUG:
            render_log_level = colored("[" + log_level.name + "]", color="blue", attrs=['blink', 'bold'])
        elif log_level == LogLevel.WARNING:
            render_log_level = colored("[" + log_level.name + "]", color="yellow", attrs=['blink', 'bold'])
        elif log_level == LogLevel.ERROR:
            render_log_level = colored("[" + log_level.name + "]", color="red", attrs=['blink', 'bold'])
        elif log_level == LogLevel.CRITICAL:
            render_log_level = colored("[" + log_level.name + "]", color="magenta", attrs=['blink', 'bold'])
        return render_log_level

    def theme_log(self, log_level, log_text):
        render_log = ""
        if log_level is LogLevel.INFO:
            render_log = colored(log_text, color="green")
        elif log_level is LogLevel.DEBUG:
            render_log = colored(log_text, color="blue")
        elif log_level is LogLevel.WARNING:
            render_log = colored(log_text, color="yellow", attrs=['bold'])
        elif log_level is LogLevel.ERROR:
            render_log = colored(log_text, color="red", attrs=['bold'])
        elif log_level is LogLevel.CRITICAL:
            render_log = colored(log_text, color="magenta", attrs=['bold'])

        return render_log



import abc


class LoggyTheme(metaclass=abc.ABCMeta):

    def __init__(self):
        pass

    @abc.abstractmethod
    def apply_theme(self, log_level, log, *texts, name=None, show_log_time=False, log_time_string=""):
        """
        Theme Designer can customize the beautiful logs in this function, Implementation details on github repo wiki
        for tutorial and api docs.
        :param log_level: enum LogLevel
        :param log: log text
        :param texts: log text list
        :param name: user can change the module_name with temporary this param
        :param show_log_time: is log output the time or not
        :param log_time_string: the log time string if log time is enable
        :return: rendered colorful log string
        """
        return NotImplemented

    @abc.abstractmethod
    def theme_name(self, log_level, name):
        """
        Render the module name to the color for output string. Implementation details on github repo wiki for tutorial
        and api docs.
        :param log_level: enum LogLevel
        :param name: log text
        :return: rendered colorful log string
        """
        return NotImplemented

    @abc.abstractmethod
    def theme_log_time(self, log_level, log_time_text):
        """
        Render the log time to the colorful output string, Implementation details on github repo wiki for tutorial and
        api docs
        :param log_level: enum LogLevel
        :param log_time_text: the log time string generate by Loggy core
        :return: rendered colorful log time string
        """
        return NotImplemented

    @abc.abstractmethod
    def theme_log_level(self, log_level):
        """
        Render the log level to the color for output string. Implementation details on github repo wiki for tutorial and
        api docs.
        :param log_level: enum LogLevel
        :return: rendered colorful log level string
        """
        return NotImplemented

    @abc.abstractmethod
    def theme_log(self, log_level, log_text):
        """
        Render the log to the color for output string. Implementation details on github repo wiki for tutorial and api
        docs.
        :param log_level: enum LogLevel
        :param log_text: log text
        :return: rendered colorful log string
        """
        return NotImplemented

    def render_name(self, log_level, name):
        """
        Render the name into colorful text. Normally you won't need to override this function
        :param log_level: log type
        :param name: name of module or user customize name
        :return: rendered name text
        """
        if name is None:
            r_name = ""
        else:
            r_name = self.theme_name(log_level, name)
        return r_name

    def render_log_time(self, log_level, log_time_string):
        """
        Render the log time into colorful string. Normally you won't need to override this function
        :param log_time_string: the log time string generate by Loggy core
        :return: render log time string
        """
        return self.theme_log_time(log_level, log_time_string)

    def render_log_level(self, log_level):
        """
        Render the log level into colorful string. Normally you won't need to override this function
        :param log_level: LogLevelEnum
        :return: rendered log level string
        """
        return self.theme_log_level(log_level)

    def render_log_text(self, log_level, log_text, *logs):
        """
        Render the log into colorful text. Normally you won't need to override this function
        :param log_level: log type
        :param log_text: log text
        :param logs: logs array
        :return: rendered log text
        """
        r_log = self.theme_log(log_level, log_text)

        for log in logs:
            r_log += self.theme_log(log_level, log)

        return r_log



import setuptools

with open("../README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="LoggyLogger",
    version="1.0.0",
    author="Eric Shih",
    author_email="eric12345566@gmail.com",
    description="Convenient, friendly and clean logger for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/eric12345566/loggy",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: System :: Logging"
    ],
    python_requires='>=3',
)


from .theme.ClassicTheme import ClassicTheme
from .module.LogLevelEnum import LogLevel
import time


class Loggy:
    _theme = ClassicTheme
    _show_log_time = True
    _log_time_struct = "%m/%d %H:%M:%S"

    def __init__(self, module_name=None, theme=None, show_log_time=None, log_time_struct=None):
        if theme is not None:
            self.__theme = theme()
        else:
            self.__theme = Loggy._theme()

        if show_log_time is not None:
            self.__show_log_time = show_log_time
        else:
            self.__show_log_time = Loggy._show_log_time

        self.__module_name = module_name

        if log_time_struct is not None:
            self.__log_time_struct = log_time_struct
        else:
            self.__log_time_struct = Loggy._log_time_struct

    @staticmethod
    def __print_log(log_text):
        print(log_text)

    def set_theme(self, loggyTheme):
        self.__theme = loggyTheme()

    def set_show_log_time(self, log_time):
        self.__show_log_time = log_time

    @classmethod
    def class_set_log_time_struct(cls, time_struct):
        cls._log_time_struct = time_struct

    @classmethod
    def class_get_log_time_struct(cls):
        return cls._log_time_struct

    @classmethod
    def class_set_show_log_time(cls, show_log_time):
        cls._show_log_time = show_log_time

    @classmethod
    def class_get_show_log_time(cls):
        return cls._show_log_time

    @classmethod
    def class_set_theme(cls, theme):
        cls._theme = theme

    @classmethod
    def class_get_theme(cls):
        return cls._theme

    def info(self, log, *texts, name=None):
        log_time = time.localtime()
        log_time_string = time.strftime(self.__log_time_struct, log_time)
        
        if name is not None:
            render_log = self.__theme.apply_theme(LogLevel.INFO, log, *texts, name=name
                                                  , show_log_time=self.__show_log_time, log_time_string=log_time_string)
        else:
            render_log = self.__theme.apply_theme(LogLevel.INFO, log, *texts, name=self.__module_name
                                                  , show_log_time=self.__show_log_time, log_time_string=log_time_string)

        self.__print_log(render_log)

    def debug(self, log, *texts, name=None):
        log_time = time.localtime()
        log_time_string = time.strftime(self.__log_time_struct, log_time)

        if name is not None:
            render_log = self.__theme.apply_theme(LogLevel.DEBUG, log, *texts, name=name
                                                  , show_log_time=self.__show_log_time, log_time_string=log_time_string)
        else:
            render_log = self.__theme.apply_theme(LogLevel.DEBUG, log, *texts, name=self.__module_name
                                                  , show_log_time=self.__show_log_time, log_time_string=log_time_string)

        self.__print_log(render_log)

    def warning(self, log, *texts, name=None):
        log_time = time.localtime()
        log_time_string = time.strftime(self.__log_time_struct, log_time)

        if name is not None:
            render_log = self.__theme.apply_theme(LogLevel.WARNING, log, *texts, name=name
                                                  , show_log_time=self.__show_log_time, log_time_string=log_time_string)
        else:
            render_log = self.__theme.apply_theme(LogLevel.WARNING, log, *texts, name=self.__module_name
                                                  , show_log_time=self.__show_log_time, log_time_string=log_time_string)

        self.__print_log(render_log)

    def error(self, log, *texts, name=None):
        log_time = time.localtime()
        log_time_string = time.strftime(self.__log_time_struct, log_time)

        if name is not None:
            render_log = self.__theme.apply_theme(LogLevel.ERROR, log, *texts, name=name
                                                  , show_log_time=self.__show_log_time, log_time_string=log_time_string)
        else:
            render_log = self.__theme.apply_theme(LogLevel.ERROR, log, *texts, name=self.__module_name
                                                  , show_log_time=self.__show_log_time, log_time_string=log_time_string)

        self.__print_log(render_log)

    def critical(self, log, *texts, name=None):
        log_time = time.localtime()
        log_time_string = time.strftime(self.__log_time_struct, log_time)

        if name is not None:
            render_log = self.__theme.apply_theme(LogLevel.CRITICAL, log, *texts, name=name
                                                  , show_log_time=self.__show_log_time, log_time_string=log_time_string)
        else:
            render_log = self.__theme.apply_theme(LogLevel.CRITICAL, log, *texts, name=self.__module_name
                                                  , show_log_time=self.__show_log_time, log_time_string=log_time_string)

        self.__print_log(render_log)

